import { defineComponent, onMounted, onBeforeUnmount, ref, watch, computed, reactive, openBlock, createElementBlock, createElementVNode, normalizeStyle, unref, Fragment, renderList, withModifiers, normalizeClass, renderSlot, toDisplayString, createVNode, createStaticVNode } from "vue";
const findTreeNode = (event2, nodeKey) => {
  let treeKey, element;
  const findParentNode = (node) => {
    if (!node || !node.getAttribute)
      return;
    const key = node.getAttribute(nodeKey);
    if (key) {
      treeKey = key;
      element = node;
      return;
    }
    findParentNode(node.parentNode);
  };
  findParentNode(event2);
  return [treeKey, element];
};
const updateChildrenVisible = (children = [], status, treeData = [], { children: childrenKey = "children", label: labelKey = "label", nodeKey = "id" }) => {
  children.forEach((node) => {
    const item = treeData.find((item2) => item2[nodeKey] === node.id);
    item.visible = status;
    if (!status)
      item.expand = status;
    if (node[childrenKey] && node[childrenKey].length && !status)
      updateChildrenVisible(node[childrenKey], status, treeData, { children: childrenKey, label: labelKey, nodeKey });
  });
};
const flattenData = (data, defaultExpandAll) => {
  const flattenFun = function(list, childKey = "children", level = 1, parent, defaultExpand = true) {
    let arr = [];
    list.forEach((element) => {
      const item = JSON.parse(JSON.stringify(element));
      item.level = level;
      if (item.expand === void 0) {
        item.expand = defaultExpand;
      }
      if (item.visible === void 0) {
        item.visible = true;
      }
      if (!parent.visible || !parent.expand) {
        item.visible = false;
      }
      item.parent = parent;
      arr.push(item);
      if (item[childKey]) {
        arr.push(...flattenFun(item[childKey], childKey, level + 1, item, defaultExpand));
      }
    });
    return arr;
  };
  return flattenFun(data, "children", 1, { level: 0, visible: true, expand: true, children: data }, defaultExpandAll);
};
var iconfont = "";
var index_vue_vue_type_style_index_0_lang = "";
const _hoisted_1$1 = ["data-tree-key"];
const _hoisted_2 = ["draggable", "data-node-key", "onContextmenu"];
const _hoisted_3 = ["onClick"];
const _hoisted_4 = { class: "qjkj-tree-node-not-expand-label" };
const _hoisted_5 = { class: "qjkj-tree-node-right" };
const _hoisted_6 = ["onClick"];
const _hoisted_7 = ["onClick"];
const _hoisted_8 = /* @__PURE__ */ createElementVNode("i", { class: "iconfont icon-vuesax-outline-edit" }, null, -1);
const _hoisted_9 = {
  key: 1,
  class: "qjkj-tree-empty"
};
const _hoisted_10 = /* @__PURE__ */ createElementVNode("hr", null, null, -1);
const _sfc_main$1 = /* @__PURE__ */ defineComponent({
  __name: "index",
  props: {
    data: {
      type: Array,
      default: []
    },
    nodeSize: {
      type: Number,
      default: 30
    },
    props: {
      type: Object,
      default: {
        children: "children",
        label: "label"
      }
    },
    nodeKey: {
      type: String,
      default: "id"
    },
    treeKey: {
      type: String,
      default: "qjkj-tree"
    },
    defaultExpandAll: {
      type: Boolean,
      default: false
    },
    draggable: {
      type: Boolean,
      default: false
    }
  },
  setup(__props, { expose }) {
    const props = __props;
    onMounted(() => {
      screenHeight.value = qjkjTreeRef.value.clientHeight;
      end.value = start.value + visibleCount.value;
      document.addEventListener("mousedown", winMousedown);
    });
    onBeforeUnmount(() => {
      document.removeEventListener("mousedown", winMousedown);
    });
    let treeData = [];
    const qjkjTreeRef = ref();
    const screenHeight = ref(0);
    const startOffset = ref(0);
    const start = ref(0);
    const end = ref();
    watch(() => props.data, (value) => {
      treeData = flattenData(value, props.defaultExpandAll);
      setFilterVisibleNode();
    }, { deep: true });
    const filterVisibleNode = ref([]);
    const listHeight = computed(() => filterVisibleNode.value.length * props.nodeSize);
    const visibleCount = computed(() => Math.ceil(screenHeight.value / props.nodeSize));
    const getTransform = computed(() => `translate3d(0, ${startOffset.value}px, 0)`);
    const visibleData = computed(() => filterVisibleNode.value.slice(start.value, Math.min(end.value, filterVisibleNode.value.length)));
    const setFilterVisibleNode = () => filterVisibleNode.value = treeData.filter((item) => item.visible);
    const scrollEvent = () => {
      const scrollTop = qjkjTreeRef.value.scrollTop;
      start.value = Math.floor(scrollTop / props.nodeSize);
      end.value = start.value + visibleCount.value;
      startOffset.value = scrollTop - scrollTop % props.nodeSize;
    };
    const handleExpand = ({ id }) => {
      const node = treeData.find((item) => item[props.nodeKey] === id);
      node.expand = !node.expand;
      if (node[props.props.children] && node[props.props.children].length)
        updateChildrenVisible(node[props.props.children], node.expand, treeData, { ...props.props, nodeKey: props.nodeKey });
      setFilterVisibleNode();
    };
    const menuStyle = reactive({ top: -9999, left: -9999 });
    const menuRow = reactive({ row: {} });
    const showMenu = (event2, row) => {
      const { x, y } = event2;
      menuRow.row = row;
      menuStyle.top = y;
      menuStyle.left = x;
    };
    const winMousedown = (event2) => {
      if (!event2.target.classList.contains("qjkj-tree-context-menu-item")) {
        hiddenMenu();
      }
    };
    const hiddenMenu = () => {
      menuRow.row = {};
      menuStyle.top = -9999;
      menuStyle.left = -9999;
    };
    const onContextMenu = (type) => {
      switch (type) {
        case "add":
          addNode(menuRow.row);
          break;
        case "delete":
          deleteNode(menuRow.row);
          break;
        case "moveUp": {
          const index2 = treeData.findIndex((item) => item[props.nodeKey] === menuRow.row[props.nodeKey]);
          if (index2 !== 0) {
            treeData[index2] = treeData.splice(index2 - 1, 1, treeData[index2])[0];
          }
          setFilterVisibleNode();
          break;
        }
        case "moveDown": {
          const index2 = treeData.findIndex((item) => item[props.nodeKey] === menuRow.row[props.nodeKey]);
          if (index2 !== treeData.length - 1) {
            treeData[index2] = treeData.splice(index2 + 1, 1, treeData[index2])[0];
          }
          setFilterVisibleNode();
          break;
        }
      }
      hiddenMenu();
    };
    const addNode = (row) => {
      onDropEnd("new node", row[props.nodeKey]);
    };
    const deleteNode = (row) => {
      const index2 = treeData.findIndex((item) => item[props.nodeKey] === row[props.nodeKey]);
      treeData.splice(index2, 1);
      setFilterVisibleNode();
    };
    const onDropEnd = (dragId, dropId) => {
      const index2 = treeData.findIndex((item) => item[props.nodeKey] === dropId);
      const element = treeData[index2];
      const newItem = {
        label: `new label ${dragId}`,
        level: element.level + 1,
        expand: true,
        visible: true,
        id: `new${dropId}`,
        parent: { ...element }
      };
      if (!element.children)
        element.children = [];
      element.children.push(newItem);
      treeData.splice(index2 + 1, 0, newItem);
      setFilterVisibleNode();
    };
    expose({ onDropEnd });
    return (_ctx, _cache) => {
      return openBlock(), createElementBlock("div", {
        ref_key: "qjkjTreeRef",
        ref: qjkjTreeRef,
        class: "qjkj-tree-box",
        "data-tree-key": props.treeKey,
        onScroll: scrollEvent
      }, [
        createElementVNode("div", {
          class: "qjkj-tree-phantom",
          style: normalizeStyle({ height: unref(listHeight) + "px" })
        }, null, 4),
        createElementVNode("ul", {
          class: "qjkj-tree",
          style: normalizeStyle({ transform: unref(getTransform) })
        }, [
          unref(visibleData) && unref(visibleData).length ? (openBlock(true), createElementBlock(Fragment, { key: 0 }, renderList(unref(visibleData), (item) => {
            return openBlock(), createElementBlock("li", {
              draggable: props.draggable,
              class: "qjkj-tree-node",
              "data-node-key": item[props.nodeKey],
              style: normalizeStyle({ height: props.nodeSize + "px" }),
              onContextmenu: withModifiers(($event) => showMenu($event, item), ["prevent"])
            }, [
              createElementVNode("div", {
                class: "qjkj-tree-node-left",
                style: normalizeStyle({ height: props.nodeSize + "px", paddingLeft: 18 * (item.level - 1) + "px" })
              }, [
                createElementVNode("i", {
                  class: normalizeClass(["qjkj-tree-node-not-expand-icon", { "qjkj-tree-node-expand-icon": item.expand, "hidden-expand-icon": !item[props.props.children] }]),
                  onClick: ($event) => handleExpand({ id: item[props.nodeKey] })
                }, null, 10, _hoisted_3),
                renderSlot(_ctx.$slots, "nodeHeader", { row: item }),
                createElementVNode("span", _hoisted_4, toDisplayString(item[props.props.label]), 1)
              ], 4),
              createElementVNode("div", _hoisted_5, [
                createElementVNode("i", {
                  class: "iconfont icon-vuesax-outline-add-square",
                  onClick: ($event) => addNode(item)
                }, null, 8, _hoisted_6),
                createElementVNode("i", {
                  class: "iconfont icon-vuesax-outline-trush-square",
                  onClick: ($event) => deleteNode(item)
                }, null, 8, _hoisted_7),
                _hoisted_8
              ])
            ], 44, _hoisted_2);
          }), 256)) : (openBlock(), createElementBlock("div", _hoisted_9, "\u6682\u65E0\u6570\u636E"))
        ], 4),
        createElementVNode("ul", {
          type: "context",
          class: "qjkj-tree-context-menu",
          style: normalizeStyle({ top: menuStyle.top + "px", left: menuStyle.left + "px" })
        }, [
          createElementVNode("li", {
            class: "qjkj-tree-context-menu-item",
            onClick: _cache[0] || (_cache[0] = ($event) => onContextMenu("add"))
          }, "\u65B0\u589E"),
          createElementVNode("li", {
            class: "qjkj-tree-context-menu-item",
            onClick: _cache[1] || (_cache[1] = ($event) => onContextMenu("edit"))
          }, "\u4FEE\u6539"),
          _hoisted_10,
          createElementVNode("li", {
            class: "qjkj-tree-context-menu-item",
            onClick: _cache[2] || (_cache[2] = ($event) => onContextMenu("delete"))
          }, "\u5220\u9664"),
          createElementVNode("li", {
            class: "qjkj-tree-context-menu-item",
            onClick: _cache[3] || (_cache[3] = ($event) => onContextMenu("select"))
          }, "\u9009\u62E9"),
          createElementVNode("li", {
            class: "qjkj-tree-context-menu-item",
            onClick: _cache[4] || (_cache[4] = ($event) => onContextMenu("moveUp"))
          }, "\u5411\u4E0A\u79FB\u52A8"),
          createElementVNode("li", {
            class: "qjkj-tree-context-menu-item",
            onClick: _cache[5] || (_cache[5] = ($event) => onContextMenu("moveDown"))
          }, "\u5411\u4E0B\u79FB\u52A8")
        ], 4)
      ], 40, _hoisted_1$1);
    };
  }
});
_sfc_main$1.install = (app) => {
  app.component(_sfc_main$1.__name, _sfc_main$1);
};
let dragStartTreeKey, dragStartNodeKey, dragStartElement;
const onDragStart = (event2) => {
  const [node, element] = findTreeNode(event2.target, "data-node-key");
  if (!node) {
    event2.dataTransfer.effectAllowed = "none";
    return;
  }
  const [tree] = findTreeNode(event2.target, "data-tree-key");
  dragStartTreeKey = tree;
  dragStartNodeKey = node;
  dragStartElement = element;
  dragStartElement.classList.add("qjkj-tree-node-drag-start", "qjkj-tree-node-drag-enter");
};
let dropElement, dropElementKey, dropTreeKey;
const onDragOver = (event2) => {
  var _a, _b;
  event2.preventDefault();
  if (!dragStartNodeKey)
    return;
  const [node, element] = findTreeNode(event2.target, "data-node-key");
  if (!element || !(element instanceof HTMLElement)) {
    event2.dataTransfer.dropEffect = "none";
    if (dropElement !== void 0) {
      if (dropElement instanceof HTMLElement)
        (_a = dropElement == null ? void 0 : dropElement.classList) == null ? void 0 : _a.remove("qjkj-tree-node-drag-enter");
      dropElement = void 0;
      dropElementKey = void 0;
    }
  } else {
    const [_tree] = findTreeNode(element, "data-tree-key");
    if (_tree === dragStartTreeKey) {
      event2.dataTransfer.dropEffect = "none";
    } else {
      if (dropElement !== element) {
        if (dropElement instanceof HTMLElement)
          (_b = dropElement == null ? void 0 : dropElement.classList) == null ? void 0 : _b.remove("qjkj-tree-node-drag-enter");
        dropElement = element;
        dropTreeKey = _tree;
        dropElementKey = node;
        dropElement.classList.add("qjkj-tree-node-drag-enter");
      }
    }
  }
};
const onDragEnd = () => {
  dragStartElement.classList.remove("qjkj-tree-node-drag-start", "qjkj-tree-node-drag-enter");
};
const onDrop = (event2, callback) => {
  var _a;
  event2.preventDefault();
  if (!dragStartNodeKey || !dropElementKey)
    return;
  if (dropElement instanceof HTMLElement)
    (_a = dropElement == null ? void 0 : dropElement.classList) == null ? void 0 : _a.remove("qjkj-tree-node-drag-enter");
  callback && callback(dragStartNodeKey, dropElementKey, dragStartTreeKey, dropTreeKey);
};
const onDraggable = (elementBody, callback) => {
  elementBody.addEventListener("dragstart", onDragStart);
  elementBody.addEventListener("dragover", onDragOver);
  elementBody.addEventListener("dragend", onDragEnd);
  elementBody.addEventListener("drop", () => onDrop(event, callback));
};
var dragTree_vue_vue_type_style_index_0_scoped_true_lang = "";
var _export_sfc = (sfc, props) => {
  const target = sfc.__vccOpts || sfc;
  for (const [key, val] of props) {
    target[key] = val;
  }
  return target;
};
const _hoisted_1 = /* @__PURE__ */ createStaticVNode('<div class="qjkj-drag-tree-transfer" data-v-aeed1ee4><button class="transfer-left transfer-btn" data-v-aeed1ee4><i class="iconfont icon-outline-arrow-up-2" data-v-aeed1ee4></i></button><button class="transfer-right transfer-btn" data-v-aeed1ee4><i class="iconfont icon-outline-arrow-up-2" data-v-aeed1ee4></i></button></div>', 1);
const _sfc_main = /* @__PURE__ */ defineComponent({
  __name: "dragTree",
  props: {
    data: {
      type: Array,
      default: []
    },
    nodeSize: {
      type: Number,
      default: 30
    },
    props: {
      type: Object,
      default: {
        children: "children",
        label: "label"
      }
    },
    nodeKey: {
      type: String,
      default: "id"
    },
    treeKey: {
      type: String,
      default: "qjkj-tree"
    },
    defaultExpandAll: {
      type: Boolean,
      default: true
    },
    draggable: {
      type: Boolean,
      default: false
    }
  },
  setup(__props) {
    const props = __props;
    const qjkeDragTreeRef = ref();
    const treeLeft = ref();
    const treeRight = ref();
    onMounted(() => {
      onDraggable(qjkeDragTreeRef.value, function(dragId, dropId, dragTree2, dropTree) {
        console.log("\u8FD4\u56DE\u4E86\uFF1F", { dragId, dropId, dragTree: dragTree2, dropTree });
        if (dropTree === "treeLeft") {
          treeLeft.value.onDropEnd(dragId, dropId);
        } else if (dropTree === "treeRight") {
          treeRight.value.onDropEnd(dragId, dropId);
        }
      });
    });
    return (_ctx, _cache) => {
      return openBlock(), createElementBlock("div", {
        class: "qjke-drag-tree",
        ref_key: "qjkeDragTreeRef",
        ref: qjkeDragTreeRef
      }, [
        createVNode(_sfc_main$1, {
          class: "qjkj-drag-tree-item",
          ref_key: "treeLeft",
          ref: treeLeft,
          data: props.data,
          treeKey: "treeLeft",
          nodeSize: props.nodeSize,
          draggable: props.draggable
        }, null, 8, ["data", "nodeSize", "draggable"]),
        _hoisted_1,
        createVNode(_sfc_main$1, {
          class: "qjkj-drag-tree-item",
          ref_key: "treeRight",
          ref: treeRight,
          data: props.data,
          treeKey: "treeRight",
          nodeSize: props.nodeSize,
          draggable: props.draggable
        }, null, 8, ["data", "nodeSize", "draggable"])
      ], 512);
    };
  }
});
var dragTree = /* @__PURE__ */ _export_sfc(_sfc_main, [["__scopeId", "data-v-aeed1ee4"]]);
dragTree.install = (app) => {
  app.component(dragTree.__name, dragTree);
};
const components = [_sfc_main$1, dragTree];
const install = (App2) => {
  components.forEach((component) => {
    App2.component(component.__name, component);
  });
};
var index = { install, ...components };
export { index as default, dragTree, _sfc_main$1 as tree };
